# coding: utf-8

ADJ_FORM = "adj_form"
ADJECTIVAL_NUMBER = "adjectival_number"
AMBIGUOUS = "ambiguous"
ANIMACY = "animacy"
ANTECEDENT_LEMMA = "antecedent_lemma"
ASPECT = "aspect"
CAPITAL = "capital"
CASE = "case"
COMPOSITE_TOKEN_LENGTH = 'composite_token_length'
COMPOSITE_TOKEN_TYPE = "composite_token_type"
COMPOSITE_TOKEN_VALUE = "composite_token_value"
DEGREE = "degree"
DEPENDENCY_TYPE = "dependency_type"
DEPENDENT_ADPOSITIONS = "dependent_adpositions"
FULL_NAMES = "full_names"
GENDER = "gender"
GRAMMEM_INFO = "grammem_info"
HEAD = "head"
IMPORTANT_SURNAME = "important_surname"
IMPORTANT_SURNAME_VALUE = "important_surname_value"
IS_BASTARD = "is_bastard"
IS_BEGINNING_OF_ANAPHOR = "is_beginning_of_anaphor"
IS_BEGINNING_OF_COMPOSITE = "is_beginning_of_composite"
IS_STOP_WORD = "is_stop_word"
LATITUDE = "latitude"
LEMMA = "lemma"
LIST_OF_DEPENDENTS = "list_of_dependents"
LIST_OF_TOKEN_TYPES_DATA = "list_of_token_types_data"
LOCALITY_TYPE = "locality_type"
LOCATIVE_VALUE = "locative_value"
LONGITUDE = "longitude"
MOOD = "mood"
NUMBER = "number"
NUMFORM = "numform"
OTHER = "other"
PART_OF_SPEECH = "part_of_speech"
PERSON = "person"
RAW_GRAM_INFO = "raw_gram_info"
SAO_TYPE = "sao_type"
TENSE = "tense"
TEXT = "text"
RAW_TEXT = 'raw_text'
TIMEZONE = "timezone"
CURRENCY = 'currency'
COUNTRY = 'country'
COUNTRY_HIDDEN = 'country_hidden'
TOKEN_TYPE = "token_type"
TOKEN_VALUE = "token_value"
TRANSITIVITY = "transitivity"
VALUE = "value"
VERBFORM = "verbform"
VOICE = "voice"
ANAPHORA_ORIGINAL_LEMMA = "anaphora_original_lemma"
W2V_STRING = "w2v_string"
ORG_TYPES = "org_types"
ORGANIZATIONS = "organizations"
START_GEO_SEQUENCE = "start_geo_sequence"
START_ORG_SEQUENCE = "start_org_sequence"
SEQUENCE_TYPE = "sequence_type"
SEQUENCE_ITEM_VALUE = "sequence_item_value"
PART_OF_SPEECH_PYMORPHY = "_POS"
