ANNOTATIONS = "annotations"
MESSAGE = "message"
APPLICATION_ID = "applicationId"
APP_INFO = "app_info"
APP_VERSION_ID = "appversionId"
AUTO_LISTENING = "auto_listening"
CLASSES = 'classes'
DEVICE = "device"
FINISHED = "finished"
FRONTEND_ENDPOINT = "frontendEndpoint"
FRONTEND_TYPE = "frontendType"
INTENT = "intent"
INTENT_META = "intent_meta"
ITEMS = "items"
MESSAGE_NAME = "messageName"
PROBAS = 'probas'
PROJECT_ID = "projectId"
PROJECT_NAME = "projectName"
PRONOUNCE_TEXT = "pronounceText"
SERVER_ACTION = "server_action"
SMART_BIO = "smartBio"
SUB = "sub"
SYSTEM_NAME = "systemName"
USER_CHANNEL = "userChannel"
USER_ID = "userId"
MESSAGE = "message"

DEBUG_INFO = "debug_info"
DEBUG_INFO_APP_KEY = "debug_info_key"
EVENTS = "events"
HISTORY_DATA = "historyData"
