ANSWER_TO_USER = "ANSWER_TO_USER"
ERROR = "ERROR"
NOTHING_FOUND = "NOTHING_FOUND"
MESSAGE_TO_SKILL = "MESSAGE_TO_SKILL"
LOCAL_TIMEOUT = "LOCAL_TIMEOUT"
RUN_APP = "RUN_APP"
CLOSE_APP = "CLOSE_APP"
SERVER_ACTION = "SERVER_ACTION"
GET_PROFILE_DATA = "GET_PROFILE_DATA"
TAKE_PROFILE_DATA = "TAKE_PROFILE_DATA"
