NUM_TOKEN = "NUM_TOKEN"
TWO_NUM_TOKENS = "{} {}".format(NUM_TOKEN, NUM_TOKEN)
CCY_TOKEN = "CCY_TOKEN"
PHONE_NUMBER_TOKEN = "PHONE_NUMBER_TOKEN"
TIME_DATE_TOKEN = "TIME_DATE_TOKEN"
TIME_YEAR_TOKEN = "TIME_YEAR_TOKEN"
TIME_MONTH_TOKEN = "TIME_MONTH_TOKEN"
CARD_NUMBER_TOKEN = "CARD_NUMBER_TOKEN"
ANIMACY_TOKEN = "ANIMACY_TOKEN"
SENTENCE_ENDPOINT_TOKEN = "SENTENCE_ENDPOINT_TOKEN"
TIME_DAY_TOKEN = "TIME_DAY_TOKEN"
TIME_TIME_TOKEN = "TIME_TIME_TOKEN"
TIME_TEMPORAL_TOKEN = "TIME_TEMPORAL_TOKEN"
NAME_TOKEN = "NAME_TOKEN"
SURNAME_TOKEN = "SURNAME_TOKEN"
PATRONYMIC_TOKEN = "PATRONYMIC_TOKEN"
EMAIL_ADDRESS_TOKEN = "EMAIL_ADDRESS_TOKEN"
URL_TOKEN = "URL_TOKEN"
EMOJI_TOKEN = "EMOJI_TOKEN"
GEO_TOKEN = "GEO_TOKEN"
CATALOG_GEO_TOKEN = "CATALOG_GEO_TOKEN"
ORG_TOKEN = "ORG_TOKEN"
CATALOG_ORG_TOKEN = "CATALOG_ORG_TOKEN"
THUMB_UP_TOKEN = "THUMB_UP_TOKEN"
THUMB_DOWN_TOKEN = "THUMB_DOWN_TOKEN"
SMS_CARDNUM_TOKEN = "SMS_CARDNUM_TOKEN"
SMS_OWNER_CARDNUM_TOKEN = "SMS_OWNER_CARDNUM_TOKEN"
SMS_KEYWORD_TOKEN = "SMS_KEYWORD_TOKEN"
SMS_ALIAS_TOKEN = "SMS_ALIAS_TOKEN"
PERSON_TOKEN = "PERSON_TOKEN"
TIME_DATE_INTERVAL_TOKEN = "TIME_DATE_INTERVAL_TOKEN"
MONEY_TOKEN = "MONEY_TOKEN"
PERIOD_TOKEN = "PERIOD_TOKEN"
RELATIVE_TIME_TOKEN = "RELATIVE_TIME_TOKEN"
