KEY_NAME = "key_name"
SCENARIO_VALUE = "scenario"
REQUIREMENT_CHECK_VALUE = "requirement_check"
REQUIREMENT_TRACE_VALUE = "requirement_trace"
TEMPLATE_TRACE_VALUE = "jinja2_trace"
EXCEPTION_VALUE = "exception"
HANDLED_EXCEPTION_VALUE = "handled_exception"
IGNITE_VALUE = "ignite"
MEMCACHED_VALUE = "memcached"
TWISTED_SERVER = "twisted"
REPOSITORY_LOAD_VALUE = "repository_load"
REPOSITORY_CLEAR_VALUE = "repository_clear"
KAFKA_ON_ASSIGN_VALUE = "kafka_on_assign"
IR_STARTUP_VALUE = "intent_recognizer_startup"
INTENT_ID_VALUE = "intent_id"
REQUEST_VALUE = "request"
