# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: SmartAppToMessage.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='SmartAppToMessage.proto',
  package='tutorial',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17SmartAppToMessage.proto\x12\x08tutorial\x1a\x1cgoogle/protobuf/struct.proto\"\xae\x02\n\x11SmartAppToMessage\x12\x11\n\tmessageId\x18\x01 \x01(\x03\x12\x11\n\tsessionId\x18\x02 \x01(\t\x12\x13\n\x0bmessageName\x18\x03 \x01(\t\x12\x39\n\x07payload\x18\x04 \x03(\x0b\x32(.tutorial.SmartAppToMessage.PayloadEntry\x12.\n\x04uuid\x18\x05 \x01(\x0b\x32 .tutorial.SmartAppToMessage.UUID\x1a\x46\n\x0cPayloadEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12%\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.Value:\x02\x38\x01\x1a+\n\x04UUID\x12\x0e\n\x06userId\x18\x01 \x01(\t\x12\x13\n\x0buserChannel\x18\x02 \x01(\t'
  ,
  dependencies=[google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,])




_SMARTAPPTOMESSAGE_PAYLOADENTRY = _descriptor.Descriptor(
  name='PayloadEntry',
  full_name='tutorial.SmartAppToMessage.PayloadEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='tutorial.SmartAppToMessage.PayloadEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='tutorial.SmartAppToMessage.PayloadEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=255,
  serialized_end=325,
)

_SMARTAPPTOMESSAGE_UUID = _descriptor.Descriptor(
  name='UUID',
  full_name='tutorial.SmartAppToMessage.UUID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='userId', full_name='tutorial.SmartAppToMessage.UUID.userId', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userChannel', full_name='tutorial.SmartAppToMessage.UUID.userChannel', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=327,
  serialized_end=370,
)

_SMARTAPPTOMESSAGE = _descriptor.Descriptor(
  name='SmartAppToMessage',
  full_name='tutorial.SmartAppToMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='messageId', full_name='tutorial.SmartAppToMessage.messageId', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sessionId', full_name='tutorial.SmartAppToMessage.sessionId', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='messageName', full_name='tutorial.SmartAppToMessage.messageName', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='payload', full_name='tutorial.SmartAppToMessage.payload', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uuid', full_name='tutorial.SmartAppToMessage.uuid', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SMARTAPPTOMESSAGE_PAYLOADENTRY, _SMARTAPPTOMESSAGE_UUID, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=68,
  serialized_end=370,
)

_SMARTAPPTOMESSAGE_PAYLOADENTRY.fields_by_name['value'].message_type = google_dot_protobuf_dot_struct__pb2._VALUE
_SMARTAPPTOMESSAGE_PAYLOADENTRY.containing_type = _SMARTAPPTOMESSAGE
_SMARTAPPTOMESSAGE_UUID.containing_type = _SMARTAPPTOMESSAGE
_SMARTAPPTOMESSAGE.fields_by_name['payload'].message_type = _SMARTAPPTOMESSAGE_PAYLOADENTRY
_SMARTAPPTOMESSAGE.fields_by_name['uuid'].message_type = _SMARTAPPTOMESSAGE_UUID
DESCRIPTOR.message_types_by_name['SmartAppToMessage'] = _SMARTAPPTOMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SmartAppToMessage = _reflection.GeneratedProtocolMessageType('SmartAppToMessage', (_message.Message,), {

  'PayloadEntry' : _reflection.GeneratedProtocolMessageType('PayloadEntry', (_message.Message,), {
    'DESCRIPTOR' : _SMARTAPPTOMESSAGE_PAYLOADENTRY,
    '__module__' : 'SmartAppToMessage_pb2'
    # @@protoc_insertion_point(class_scope:tutorial.SmartAppToMessage.PayloadEntry)
    })
  ,

  'UUID' : _reflection.GeneratedProtocolMessageType('UUID', (_message.Message,), {
    'DESCRIPTOR' : _SMARTAPPTOMESSAGE_UUID,
    '__module__' : 'SmartAppToMessage_pb2'
    # @@protoc_insertion_point(class_scope:tutorial.SmartAppToMessage.UUID)
    })
  ,
  'DESCRIPTOR' : _SMARTAPPTOMESSAGE,
  '__module__' : 'SmartAppToMessage_pb2'
  # @@protoc_insertion_point(class_scope:tutorial.SmartAppToMessage)
  })
_sym_db.RegisterMessage(SmartAppToMessage)
_sym_db.RegisterMessage(SmartAppToMessage.PayloadEntry)
_sym_db.RegisterMessage(SmartAppToMessage.UUID)


_SMARTAPPTOMESSAGE_PAYLOADENTRY._options = None
# @@protoc_insertion_point(module_scope)
